%% Array Time Evolution
function [Cluster_Map_tplus,Cmatrix_rx_tplus,Cmatrix_tx_tplus,phasetable_tplus]=ArrayTimeEvolution(Cluster_Map_t,Cmatrix_rx_t,Cmatrix_tx_t,para,phasetable_t)
Cluster_Map_tplus=Cluster_Map_t;
Cmatrix_rx_tplus=Cmatrix_rx_t;
Cmatrix_tx_tplus=Cmatrix_tx_t;
phasetable_tplus=phasetable_t;
%% Cluster Evolution on the Time Axis
% min index of cluster index
Min_ind_rx = find((sum(Cmatrix_rx_t))>0, 1, 'last');
Min_ind_tx = find((sum(Cmatrix_tx_t))>0, 1, 'last');
Min_ind=max(Min_ind_tx,Min_ind_rx);

% read total cluster set
Cluster_set_total=unique(Cluster_Map_t(:,1));

PR=exp(-para.lambdaR*para.dT*(para.v_R+para.v_T+para.v_c*para.P_F)/para.Corr_distance);            % reconbination probability

N_new_avg=para.lambdaG/para.lambdaR*(1-PR);                       % average number of newly generated clusters

Survived_Cluster_set=Cluster_set_total.*(rand(size(Cluster_set_total))<PR);    % recombination process

Disappeared_Cluster_set=find(Survived_Cluster_set==0);
Cmatrix_rx_tplus(:,Disappeared_Cluster_set)=0;
Cmatrix_tx_tplus(:,Disappeared_Cluster_set)=0;

for k1=1:size(Disappeared_Cluster_set,1)
    
    Disappeared_Cluster_ind=find((Cluster_Map_tplus(:,1)==Disappeared_Cluster_set(k1)));    
    Cluster_Map_tplus(Disappeared_Cluster_ind,:)=[];
    phasetable_tplus(Disappeared_Cluster_ind,:)=[];
end


%% Survived Clusters Geometrical Relationship Update
% [1C_number_Tx  2Delay 3Power 4AAoA 5AAoD 6EAoA 7EAoD 8D_n_R 9D_n_T 10meanAAoA 11meanAAoD 12meanEAoA 13meanEAoD 14ClusterTvE 15ClusterTvA 16ClusterRvE 17ClusterRvA 18meanDelay]
% Calculate vc
ClusterTvE=Cluster_Map_tplus(:,14);
ClusterTvA=Cluster_Map_tplus(:,15);
ClusterRvE=Cluster_Map_tplus(:,16);
ClusterRvA=Cluster_Map_tplus(:,17);
vcR=para.v_c.*[cos(ClusterRvE).*cos(ClusterRvA) cos(ClusterRvE).*sin(ClusterRvA) sin(ClusterRvE)];
vcT=para.v_c.*[cos(ClusterTvE).*cos(ClusterTvA) cos(ClusterTvE).*sin(ClusterTvA) sin(ClusterTvE)];

 % Calculate Dn and update
 meanAAoA=Cluster_Map_tplus(:,10);
 meanAAoD=Cluster_Map_tplus(:,11);
 meanEAoA=Cluster_Map_tplus(:,12);
 meanEAoD=Cluster_Map_tplus(:,13);
 DR_n=kron(ones(1,3),Cluster_Map_tplus(:,8)).*[cos(meanEAoA).*cos(meanAAoA) cos(meanEAoA).*sin(meanAAoA) sin(meanEAoA) ];
 DT_n=kron(ones(1,3),Cluster_Map_tplus(:,9)).*[cos(meanEAoD).*cos(meanAAoD) cos(meanEAoD).*sin(meanAAoD) sin(meanEAoD)];
 DR_n_plus=DR_n+vcR*para.dT;          % Dn(t+dt)=Dn(t)+vt     
 DT_n_plus=DT_n+vcT*para.dT;          % Dn(t+dt)=Dn(t)+vt
 
% Cartisian to Sphere
[DR_n_plus_azimuth,DR_n_plus_elevation,DR_n_plus_r] = cart2sph(DR_n_plus(:,1),DR_n_plus(:,2),DR_n_plus(:,3));
Cluster_Map_tplus(:,4)=Cluster_Map_tplus(:,4)-Cluster_Map_tplus(:,10)+DR_n_plus_azimuth;
Cluster_Map_tplus(:,6)=Cluster_Map_tplus(:,6)-Cluster_Map_tplus(:,12)+DR_n_plus_elevation;
Cluster_Map_tplus(:,10)=DR_n_plus_azimuth;
Cluster_Map_tplus(:,12)=DR_n_plus_elevation;
 
[DT_n_plus_azimuth,DT_n_plus_elevation,DT_n_plus_r] = cart2sph(DT_n_plus(:,1),DT_n_plus(:,2),DT_n_plus(:,3));
Cluster_Map_tplus(:,5)=Cluster_Map_tplus(:,5)-Cluster_Map_tplus(:,11)+DT_n_plus_azimuth;
Cluster_Map_tplus(:,7)=Cluster_Map_tplus(:,7)-Cluster_Map_tplus(:,13)+DT_n_plus_elevation;
Cluster_Map_tplus(:,11)=DT_n_plus_azimuth;
Cluster_Map_tplus(:,13)=DT_n_plus_elevation;

% calculate cluster delays and powers
Survived_Cluster_set_total=unique(Cluster_Map_tplus(:,1));
for k1=1:size(Survived_Cluster_set_total,1)
    Survived_Cluster_ind=(Cluster_Map_tplus(:,1)==Survived_Cluster_set_total(k1));
    Delay=Cluster_Map_tplus(Survived_Cluster_ind,18);
    Power_t=Cluster_Map_tplus(Survived_Cluster_ind,3);
    
    Delay_t=Delay+(Cluster_Map_tplus(Survived_Cluster_ind,8)+Cluster_Map_tplus(Survived_Cluster_ind,9))/para.c;
    
    % delay tilder
    Delay_tilder=-para.DS_r*para.ds_sigma*log(rand(1));
    Delay_tilder=exp(-para.dT/para.T_Corr_distance)*Delay+(1-exp(-para.dT/para.T_Corr_distance))*Delay_tilder;
    
    % update mean delay
    Cluster_Map_tplus(Survived_Cluster_ind,18)=Delay_tilder;
    % delay(t+dt)
    Delay_tplus=Delay_tilder+(DR_n_plus_r(Survived_Cluster_ind)+DT_n_plus_r(Survived_Cluster_ind))/para.c;
    % update power
    Power_tplus=Power_t.*(Cluster_Map_tplus(Survived_Cluster_ind,2)-2*(Delay_tplus-Delay_t))./Cluster_Map_tplus(Survived_Cluster_ind,2);
    Cluster_Map_tplus(Survived_Cluster_ind,3)=max(Power_tplus,0);
    % update delay
    Cluster_Map_tplus(Survived_Cluster_ind,2)=Cluster_Map_tplus(Survived_Cluster_ind,2)-Delay+Delay_tilder;

end
% update distance
Cluster_Map_tplus(:,8)=DR_n_plus_r;
Cluster_Map_tplus(:,9)=DT_n_plus_r;

%% New Clusters Generation
N_new=poissrnd(N_new_avg);
if para.Polarization_Rx
    PR_rx=exp(-para.lambdaR*para.delta_rx/2*para.lambda/para.A_Corr_distance);            % reconbination probability
else
    PR_rx=exp(-para.lambdaR*para.delta_rx*para.lambda/para.A_Corr_distance);            % reconbination probability
end
if para.Polarization_Tx
    PR_tx=exp(-para.lambdaR*para.delta_tx/2*para.lambda/para.A_Corr_distance);            % reconbination probability
else
    PR_tx=exp(-para.lambdaR*para.delta_tx*para.lambda/para.A_Corr_distance);            % reconbination probability
end
if N_new>0
    Cmatrix_rx_tplus=[Cmatrix_rx_tplus zeros(para.M_R,N_new)];
    Cmatrix_tx_tplus=[Cmatrix_tx_tplus zeros(para.M_T,N_new)];
end
for k1=1:N_new
    % generate intial attachment at the Rx
    l_rx=randi(para.M_R);
    Cmatrix_rx_tplus(l_rx,Min_ind+k1)=1;
    % generate intial attachment at the Tx
    l_tx=randi(para.M_T);
    Cmatrix_tx_tplus(l_tx,Min_ind+k1)=1;
    k2=l_rx-1;
    while (k2>=1&&rand(1)<PR_rx)        
        Cmatrix_rx_tplus(k2,Min_ind+k1)=1;
        k2=k2-1;
    end
    k2=l_rx+1;
    while (k2<=para.M_R&&rand(1)<PR_rx)
        Cmatrix_rx_tplus(k2,Min_ind+k1)=1;
        k2=k2+1;
        
    end
    
    k2=l_tx-1;
    while (k2>=1&&rand(1)<PR_tx)        
        Cmatrix_tx_tplus(k2,Min_ind+k1)=1;
        k2=k2-1;
    end
    k2=l_tx+1;
    while (k2<=para.M_T&&rand(1)<PR_tx)
        Cmatrix_tx_tplus(k2,Min_ind+k1)=1;
        k2=k2+1;
        
    end
    
    % generate cluster parameters
    % [1C_number_Tx  2Delay 3Power 4AAoA 5AAoD 6EAoA 7EAoD 8D_n_R 9D_n_T 10meanAAoA 11meanAAoD 12meanEAoA 13meanEAoD 14ClusterTvE 15ClusterTvA 16ClusterRvE 17ClusterRvA 18meanDelay]
    New_Cluster_Map=zeros(1,18);
    New_Cluster_Map(1,1)=Min_ind+k1;
    Delay=-para.DS_r*para.ds_sigma*log(rand(1));
    New_Cluster_Map(1,2)=Delay;
    P=exp(-Delay*(para.DS_r-1)/para.DS_r/para.ds_sigma).*10.^(-para.Per_cluster_shadowing*randn(1)/10);
    New_Cluster_Map(1,3)=P;
    AAoA=(para.AAoA_variance*randn(1))/180*pi+para.nu_A_R;      % Azimuth angle Rx
    AAoD=(para.AAoD_variance*randn(1))/180*pi+para.nu_A_T;      % Azimuth angle Rx
    EAoA=(para.EAoA_variance*randn(1))/180*pi+para.nu_E_R;      % Azimuth angle Rx
    EAoD=(para.EAoD_variance*randn(1))/180*pi+para.nu_E_T;      % Azimuth angle Rx
    
%     EAoA=(para.EAoA_variance*0)/180*pi+0;      % Azimuth angle Rx
%     EAoD=(para.EAoD_variance*0)/180*pi+0;      % Azimuth angle Rx
    %generate cluster distance
    D_n_R=exprnd(para.d_Rx_mean,1,1)+para.d_Rx_min;
    D_n_T=exprnd(para.d_Tx_mean,1,1)+para.d_Tx_min;
    
    New_Cluster_Map(4:18)=[AAoA AAoD EAoA EAoD D_n_R D_n_T AAoA AAoD EAoA EAoD...
        pi*(rand(1,1)-0.5) 2*pi*(rand(1,1)-0.5) pi*(rand(1,1)-0.5) 2*pi*(rand(1,1)-0.5) Delay];
    Subpath_numbers=max(poissrnd(para.mean_num_subpath),1);
    subpath_angular_offset=pi/180*MEA_Laplace_offset_cal(Subpath_numbers,para.angular_variance_subpath).';
    subpath_delay_offset=exprnd(para.mean_delay_subpath,Subpath_numbers,1);
    subpath_delay_offset=subpath_delay_offset-min(subpath_delay_offset);
    % generate subpath power
    subpath_power=exp(-subpath_delay_offset*(para.DS_r-1)/para.mean_delay_subpath).*10.^(-para.Per_cluster_shadowing*randn(Subpath_numbers,1)/10);
    subpath_power=P*subpath_power./sum(subpath_power);
    tmp_subpath_properties=kron(ones(Subpath_numbers,1),New_Cluster_Map);
    tmp_subpath_properties(:,2)=tmp_subpath_properties(:,2)+subpath_delay_offset;
    tmp_subpath_properties(:,3)=subpath_power;
    tmp_subpath_properties(:,4:7)=tmp_subpath_properties(:,4:7)+kron(ones(1,4),subpath_angular_offset);
    Cluster_Map_tplus=[Cluster_Map_tplus;tmp_subpath_properties];
    tmp_phase=exp(1i*rand(size(tmp_subpath_properties,1),4));
    phasetable_tplus=[phasetable_tplus;[(Min_ind+k1)*ones(size(tmp_subpath_properties,1),1) tmp_phase]];
end
if para.Polarization_Rx
    Cmatrix_rx_tplus(2:2:end,:)=Cmatrix_rx_tplus(1:2:end,:);
end
if para.Polarization_Tx
    Cmatrix_tx_tplus(2:2:end,:)=Cmatrix_tx_tplus(1:2:end,:);
end
clear phasetable_t;

